
(function (Drupal) {
  Drupal.behaviors.elevenlabsJsPlayer = {
    players: {},
    intervals: {},
    timings: {},
    currentCandidate: {},
    nextCandidate: {},
    attach: function (context, settings) {
      // We always take over the play buttons click.
      context.querySelectorAll('.elevenlabs-global-play').forEach((item) => {
        let id = item.id;
        let playerData = drupalSettings.elevenlabs[id];
        // Wait until all audios are loaded, before allowing to play them.
        let counter = 0;
        if (playerData.state == 'loading') {
          playerData.state = 'stopped';
          for (let a of playerData.items) {
            document.getElementById(a.id).addEventListener('canplaythrough', () => {
              counter++;
              if (counter == playerData.totalItems) {
                document.getElementById(id).removeAttribute('disabled');
                this.calculateTimings(id);
                this.setEvents(id, item, playerData);
              }
            });
          }
        }
      })
    },
    setEvents(id, item, playerData) {
      // Play button.
      if (!item.getAttribute('listener')) {
        item.setAttribute('listener', 'true');
        item.addEventListener('click', (e) => {
          if (playerData.state == 'stopped') {
            this.startPlaying(id, item, playerData);
          } else {
            this.pausePlaying(id, item, playerData);
          }
          return false;
        });
      }
      // Seeker.
      let seeker = document.querySelector('#' + id).parentElement.querySelector('#seek-slider');
      if (!seeker.getAttribute('listener')) {
        seeker.setAttribute('listener', 'true');
        seeker.addEventListener('input', (e) => {
          this.stopAllPlayers(id, item, playerData);
          clearInterval(this.intervals[id]);
          playerData.currentTime = seeker.value * (playerData.totalTime/100);
          this.playTrack(id);
        });
      }
    },
    stopAllPlayers(id, item, playerData) {
      for (let a of playerData.items) {
        document.getElementById(a.id).pause();
      }
    },
    startPlaying(id, item, playerData) {
      item.classList.add('pause');
      playerData.state = 'started';
      if ('id' in this.currentCandidate) {
        document.getElementById(this.currentCandidate.id).play();
      }
      this.playTrack(id);
    },
    pausePlaying(id, item, playerData) {
      playerData.state = 'stopped';
      document.getElementById(this.currentCandidate.id).pause();
      this.stopTrack(id);
      item.classList.remove('pause');
    },
    playTrack(id) {
      this.getNextCandidate(id);
      this.intervals[id] = setInterval(() => {
        Drupal.behaviors.elevenlabsJsPlayer.updateTimings(id);
      }, 10);
      this.updateTimings(id)
    },
    stopTrack(id) {
      clearInterval(this.intervals[id]);
    },
    updateTimings(id) {
      let playerData = drupalSettings.elevenlabs[id];
      if ('startTime' in this.nextCandidate &&
        playerData.currentTime >= this.nextCandidate.startTime) {
        document.getElementById(this.nextCandidate.id).play();
        this.getNextCandidate(id);
        this.currentCandidate = this.nextCandidate;
      }
      playerData.currentTime += 10;
      if (playerData.currentTime >= playerData.totalTime) {
        this.resetPlayer(id);
      }
      if (playerData.currentTime % 1000 === 0) {
        document.querySelector('#audio-player-container #current-time').innerHTML = (this.calculateTime((playerData.currentTime / 1000)));
      }
    },
    calculateTimings(id) {
      let playerData = drupalSettings.elevenlabs[id];
      if (!(id in this.timings)) {
        this.timings[id] = {};
        let startTime = 0;
        for (let a of playerData.items) {
          let audio = document.getElementById(a.id);
          a.duration = audio.duration*1000;
          a.startTime = startTime;
          startTime += a.duration;
        }
        playerData.totalTime = startTime;
        document.querySelector('#audio-player-container #duration').innerHTML = (this.calculateTime((startTime/1000)));
      }
    },
    getNextCandidate(id) {
      let playerData = drupalSettings.elevenlabs[id];
      console.log(playerData);
      for (let a of playerData.items) {
        if (playerData.currentTime <= a.startTime &&
          this.currentCandidate != a) {
          this.nextCandidate = a;
          return;
        }
      }
    },
    resetPlayer(id) {
      let playerData = drupalSettings.elevenlabs[id];
      clearInterval(this.intervals[id]);
      playerData.currentTime = 0;
      this.nextCandidate = {};
      this.currentCandidate = {};
      playerData.state = 'stopped'
    },
    calculateTime(secs) {
      const minutes = Math.floor(secs / 60);
      const seconds = Math.floor(secs % 60);
      const returnedSeconds = seconds < 10 ? `0${seconds}` : `${seconds}`;
      return `${minutes}:${returnedSeconds}`;
    }
  };
})(Drupal);
